#include <Control.h>
#include <Window.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Application.h>
#include <TranslationUtils.h>
#include <AppFileInfo.h>
#include <string.h>
#include <stdio.h>
#include "LaunchBarView.h"
#include "LaunchBar.h"

LaunchBarView::LaunchBarView(BRect workRect, uint32 b_right, uint32 b_bottom, char* adresses[], int num, char* type)
	:BControl(workRect, "display", NULL, NULL, B_FOLLOW_NONE, B_WILL_DRAW| B_FRAME_EVENTS |
	B_SUBPIXEL_PRECISE)
{
	SetAppBitmap(type, workRect);
	margin = 19;
	ConstructIcons(adresses, num, type);
	isOnClick = false;
	right = b_right;
	bottom = b_bottom;
}

void LaunchBarView::Draw(BRect updateRect)
{
	DrawBitmap(back);
	BView::Draw(updateRect);
}

void LaunchBarView::MouseDown(BPoint downPoint)
{	
	uint32 mouse_button;
	
	SetMouseEventMask(B_POINTER_EVENTS, B_LOCK_WINDOW_FOCUS | B_NO_POINTER_HISTORY);
	SetTracking(true);
	
	GetMouse(&wherePoint, &mouse_button);
	switch(mouse_button)
	{
		case B_PRIMARY_MOUSE_BUTTON :
		{
			if(!((Window()) -> IsActive()))
				(Window()) -> Activate();
			
			if(!isOnClick)
				isOnClick = true;
			break;
		}
		
		case B_SECONDARY_MOUSE_BUTTON :
		{
			ShowContextMenu(ConvertToScreen(wherePoint));
			break;
		}
	}
}

void LaunchBarView::MouseUp(BPoint upPoint)
{
	isOnClick = false;
	SetTracking(false);
}

void LaunchBarView::MouseMoved(BPoint movePoint, uint32 state, const BMessage* message)
{
	goPoint=ConvertToScreen(movePoint-wherePoint);
	if (isOnClick)
	{
		if(goPoint.x<=0) goPoint.x=0;
		if(goPoint.y<=0) goPoint.y=0;
		if(goPoint.x >= (right - background.width)) goPoint.x=right- background.width;
		if(goPoint.y >= (bottom - background.height)) goPoint.y=bottom- background.height;
		(Window()) -> MoveTo(goPoint);
	}
}

void LaunchBarView::ShowContextMenu(BPoint goPoint)
{
	BMenuItem* selected;
	context_menu = new BPopUpMenu("Context Menu");
		reload_item = new BMenuItem(SpTranslate("Reload"), new BMessage(B_RELOAD_APP));
		
		quit_item = new BMenuItem(SpTranslate("Quit LaunchBar"), new BMessage(B_QUIT_REQUESTED));
	context_menu -> AddItem(reload_item);
	((LaunchBar*)be_app)->AddLanguageMenuItem(context_menu);
	context_menu->AddSeparatorItem();
	context_menu -> AddItem(quit_item);
	selected = context_menu -> Go(goPoint, true);
	
	if(selected)
		be_app -> PostMessage(selected->Message(), 0);
}

void LaunchBarView::ConstructIcons(char* paths[], int num_signatures, char* type)
{
	char button_images[2][3][20] = {{"images/Normal", "images/Over", "images/Down"} ,
						{"images/NormalBig", "images/OverBig", "images/DownBig"}};
	char* p_icon, * set_icon;
	icon_size i_size = B_MINI_ICON; int step = 0, space = 22, width = 16, top = 24, begin = 276, left = 7, b_image = 0;
	if(strcmp(type, "big_size") != 0)
	{
		icon = new BBitmap(BRect(0,0,15,15), B_RGB32);
		for(int i = 0; i != 3; i++) button_bitmaps[i] = new BBitmap(BRect(0,0,20,20), B_RGB32);
	}
	else
	{
		icon = new BBitmap(BRect(0,0,31,31), B_RGB32);
		for(int i = 0; i != 3; i++) button_bitmaps[i] = new BBitmap(BRect(0,0,41,41), B_RGB32);
		space = 44; width = 32; top = 48; begin = 1080; left = 14;
		b_image = 1; i_size = B_LARGE_ICON;
	}
	for(int l = 0; l != num_signatures; l++)
	{
		if(strcmp(paths[l], "Separator") == 0) margin += 10;
		else
		{
			app_file = new BFile(paths[l], B_READ_ONLY);
			BAppFileInfo file_info(app_file);
			file_info.GetSignature(signatures[l]);
			file_info.GetIcon(icon, i_size);
			
			for(int i = 0; i != 3; i++) button_bitmaps[i] = BTranslationUtils::GetBitmap(button_images[b_image][i], NULL);
			
			for(int o = 0; o != 3; o++)
			{
				set_icon = (char*)button_bitmaps[o] -> Bits();
				p_icon = (char*)icon -> Bits();
				for(int i = 0; i != begin; i++) set_icon += 1;
				for(int i = 0; i != width; i++)
				{
					for(int j = 0; j != width; j++)
					{				
						if((*p_icon != -1)||(*(p_icon+1) != -1)||(*(p_icon+2) != -1)||(*(p_icon+3) != -1))
							for(int n = 0; n != 4; n++) *set_icon++ = *p_icon++;
						else
						{
							set_icon += 4;
							p_icon += 4;
						}
					}
					
					for(int k = 0; k != top; k++) set_icon += 1;
				}
			}
		
			test_view[l] = new IconView(BRect(left, (margin+space*step), space + left - 1, space - 1 + margin+space*step), button_bitmaps[0], button_bitmaps[1], button_bitmaps[2], signatures[l]);
			test_view[l] -> AttachedToWindow();
			AddChild(test_view[l]);
			step += 1;
		}
	}
}	

void LaunchBarView::SetAppBitmap(char* type, BRect workRect)
{
	char image[2][21] = { "images/Background", "images/BackgroundBig"}; int num = 0;
	background.width = 35; background.height = 350;
	if(strcmp(type, "big_size") == 0)
	{
		num = 1;
		background.width = 70;
		background.height = 600;
	}
	back = new BBitmap(workRect, B_RGB32);
	back = BTranslationUtils::GetBitmap(image[num], NULL);
}

void LaunchBarView::AttachedToWindow()
{
	BView::AttachedToWindow();
}

LaunchBarView::~LaunchBarView()
{

}